# -*- coding: utf-8 -*-
# vim: autoindent shiftwidth=4 expandtab textwidth=80 tabstop=4 softtabstop=4

###############################################################################
# OpenLP - Open Source Lyrics Projection                                      #
# --------------------------------------------------------------------------- #
# Copyright (c) 2008-2010 Raoul Snyman                                        #
# Portions copyright (c) 2008-2010 Tim Bentley, Jonathan Corwin, Michael      #
# Gorven, Scott Guerrieri, Christian Richter, Maikel Stuivenberg, Martin      #
# Thompson, Jon Tibble, Carsten Tinggaard                                     #
# --------------------------------------------------------------------------- #
# This program is free software; you can redistribute it and/or modify it     #
# under the terms of the GNU General Public License as published by the Free  #
# Software Foundation; version 2 of the License.                              #
#                                                                             #
# This program is distributed in the hope that it will be useful, but WITHOUT #
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or       #
# FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for    #
# more details.                                                               #
#                                                                             #
# You should have received a copy of the GNU General Public License along     #
# with this program; if not, write to the Free Software Foundation, Inc., 59  #
# Temple Place, Suite 330, Boston, MA 02111-1307 USA                          #
###############################################################################

import logging

from common import parse_reference
from opensong import OpenSongBible
from osis import OSISBible
from csvbible import CSVBible
from db import BibleDB
from http import HTTPBible

log = logging.getLogger(__name__)

class BibleMode(object):
    """
    This is basically an enumeration class which specifies the mode of a Bible.
    Mode refers to whether or not a Bible in OpenLP is a full Bible or needs to
    be downloaded from the Internet on an as-needed basis.
    """
    Full = 1
    Partial = 2


class BibleFormat(object):
    """
    This is a special enumeration class that holds the various types of Bibles,
    plus a few helper functions to facilitate generic handling of Bible types
    for importing.
    """
    Unknown = -1
    OSIS = 0
    CSV = 1
    OpenSong = 2
    WebDownload = 3

    @staticmethod
    def get_class(id):
        """
        Return the appropriate imeplementation class.

        ``id``
            The Bible format.
        """
        if id == BibleFormat.OSIS:
            return OSISBible
        elif id == BibleFormat.CSV:
            return CSVBible
        elif id == BibleFormat.OpenSong:
            return OpenSongBible
        elif id == BibleFormat.WebDownload:
            return HTTPBible
        else:
            return None

    @staticmethod
    def list():
        """
        Return a list of the supported Bible formats.
        """
        return [
            BibleFormat.OSIS,
            BibleFormat.CSV,
            BibleFormat.OpenSong,
            BibleFormat.WebDownload
        ]


class BibleManager(object):
    """
    The Bible manager which holds and manages all the Bibles.
    """
    log.info(u'Bible manager loaded')

    def __init__(self, parent, config):
        """
        Finds all the bibles defined for the system and creates an interface
        object for each bible containing connection information. Throws
        Exception if no Bibles are found.

        Init confirms the bible exists and stores the database path.

        ``config``
            The plugin's configuration object.
        """
        log.debug(u'Bible Initialising')
        self.config = config
        self.parent = parent
        self.web = u'Web'
        self.db_cache = None
        self.path = self.config.get_data_path()
        self.proxy_name = self.config.get_config(u'proxy name')
        self.suffix = u'sqlite'
        self.import_wizard = None
        self.reload_bibles()
        self.media = None

    def reload_bibles(self):
        """
        Reloads the Bibles from the available Bible databases on disk. If a web
        Bible is encountered, an instance of HTTPBible is loaded instead of the
        BibleDB class.
        """
        log.debug(u'Reload bibles')
        files = self.config.get_files(self.suffix)
        log.debug(u'Bible Files %s', files)
        self.db_cache = {}
        for filename in files:
            bible = BibleDB(self.parent, path=self.path, file=filename,
                            config=self.config)
            name = bible.get_name()
            log.debug(u'Bible Name: "%s"', name)
            self.db_cache[name] = bible
            # look to see if lazy load bible exists and get create getter.
            source = self.db_cache[name].get_meta(u'download source')
            if source:
                download_name = self.db_cache[name].get_meta(u'download name').value
                meta_proxy = self.db_cache[name].get_meta(u'proxy url')
                web_bible = HTTPBible(self.parent, path=self.path,
                    file=filename, config=self.config,
                    download_source=source.value, download_name=download_name)
                if meta_proxy:
                    web_bible.set_proxy_server(meta_proxy.value)
                self.db_cache[name] = web_bible
        log.debug(u'Bibles reloaded')

    def set_process_dialog(self, wizard):
        """
        Sets the reference to the dialog with the progress bar on it.

        ``dialog``
            The reference to the import wizard.
        """
        self.import_wizard = wizard

    def import_bible(self, type, **kwargs):
        """
        Register a bible in the bible cache, and then import the verses.

        ``type``
            What type of Bible, one of the ``BibleFormat`` values.

        ``**kwargs``
            Keyword arguments to send to the actual importer class.
        """
        class_ = BibleFormat.get_class(type)
        kwargs['path'] = self.path
        kwargs['config'] = self.config
        importer = class_(self.parent, **kwargs)
        name = importer.register(self.import_wizard)
        self.db_cache[name] = importer
        return importer

    def get_bibles(self):
        """
        Returns a list of the names of available Bibles.
        """
        log.debug(u'get_bibles')
        return [name for name, bible in self.db_cache.iteritems()]

    def get_books(self, bible):
        """
        Returns a list of Bible books, and the number of chapters in that book.

        ``bible``
            Unicode. The Bible to get the list of books from.
        """
        log.debug(u'BibleManager.get_books("%s")', bible)
        return [
            {
                u'name': book.name,
                u'chapters': self.db_cache[bible].get_chapter_count(book.name)
            }
            for book in self.db_cache[bible].get_books()
        ]

    def get_chapter_count(self, bible, book):
        """
        Returns the number of Chapters for a given book
        """
        log.debug(u'get_book_chapter_count %s', book)
        return self.db_cache[bible].get_chapter_count(book)

    def get_verse_count(self, bible, book, chapter):
        """
        Returns all the number of verses for a given
        book and chapterMaxBibleBookVerses
        """
        log.debug(u'BibleManager.get_verse_count("%s", "%s", %s)', bible, book, chapter)
        return self.db_cache[bible].get_verse_count(book, chapter)

    def get_verses(self, bible, versetext):
        """
        Parses a scripture reference, fetches the verses from the Bible
        specified, and returns a list of ``Verse`` objects.

        ``bible``
            Unicode. The Bible to use.

        ``versetext``
            Unicode. The scripture reference. Valid scripture references are:

                - Genesis 1:1
                - Genesis 1:1-10
                - Genesis 1:1-2:10
        """
        log.debug(u'BibleManager.get_verses("%s", "%s")', bible, versetext)
        reflist = parse_reference(versetext)
        return self.db_cache[bible].get_verses(reflist)

    def save_meta_data(self, bible, version, copyright, permissions):
        """
        Saves the bibles meta data
        """
        log.debug(u'save_meta data %s,%s, %s,%s',
            bible, version, copyright, permissions)
        self.db_cache[bible].create_meta(u'Version', version)
        self.db_cache[bible].create_meta(u'Copyright', copyright)
        self.db_cache[bible].create_meta(u'Permissions', permissions)

    def get_meta_data(self, bible, key):
        """
        Returns the meta data for a given key
        """
        log.debug(u'get_meta %s,%s', bible, key)
        return self.db_cache[bible].get_meta(key)

    def exists(self, name):
        """
        Check cache to see if new bible
        """
        if not isinstance(name, unicode):
            name = unicode(name)
        for bible in self.db_cache.keys():
            log.debug(u'Bible from cache in is_new_bible %s', bible)
            if not isinstance(bible, unicode):
                bible = unicode(bible)
            if bible == name:
                return True
        return False

